<?php
/**
 * Mopple_Driver:: defines an API for implementing storage backends for Mopple.
 *
 * $Horde$
 *
 * Copyright 2004-2005 Your Name <you@example.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Your Name <you@example.com>
 * @since   Mopple 0.1
 * @package Mopple
 */
class Mopple_Driver {

    /**
     * Array holding the current MP3 list. Each array entry is a hash
     * describing a MP3. The array is indexed by the IDs.
     *
     * @var array $_mp3s
     */
    var $_mp3s = array();

    /**
     * Lists all MP3s.
     *
     * @return array  Returns a list of all MP3s.
     */
    function listMP3s()
    {
        return $this->_mp3s;
    }

    /**
     * Retrieves the MP3s from the backend.
     *
     * @abstract
     *
     * @return boolean|PEAR_Error  True on success, PEAR_Error on failure.
     */
    function retrieve()
    {
    }

    /**
     * Attempts to return a concrete Mopple_Driver instance based on $driver.
     *
     * @param string $driver  The type of the concrete Mopple_Driver subclass
     *                        to return.  The class name is based on the
     *                        storage driver ($driver).  The code is
     *                        dynamically included.
     *
     * @param array $params   A hash containing any additional configuration
     *                        or connection parameters a subclass might need.
     *
     * @return mixed  The newly created concrete Mopple_Driver instance, or
     *                false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Mopple_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Mopple_Driver instance
     * based on $driver. It will only create a new instance if no
     * Mopple_Driver instance with the same parameters currently exists.
     *
     * This method must be invoked as: $var = &Mopple_Driver::singleton()
     *
     * @param string $driver  The type of concrete Mopple_Driver subclass to
     *                        return.  The is based on the storage driver
     *                        ($driver).  The code is dynamically included.
     * @param array $params   A hash containing any additional configuration
     *                        or connection parameters a subclass might need.
     *
     * @return mixed  The created concrete Mopple_Driver instance, or false
     *                on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Mopple_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
