<?php

define('MOPPLE_VFS', '.horde/mopple/mp3s');

/**
 * Mopple Base Class.
 *
 * $Horde$
 *
 * Copyright 2004-2005 Your Name <you@example.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Your Name <you@example.com>
 * @since   Mopple 0.1
 * @package Mopple
 */
class Mopple {

    /**
     * Adds a file to Mopple's VFS backend.
     *
     * @param int $id      The MP3 ID.
     * @param array $file  A hash with the file information.
     *
     * @return mixed  A hash with MP3ID1 tag information or a PEAR_Error on
     *                failure.
     */
    function &addFile($id, $file)
    {
        global $conf;

        /* Check if this is really a MP3 file. */
        require_once 'Horde/MIME/Magic.php';
        if (MIME_Magic::analyzeFile($file['tmp_name']) != 'audio/mpeg') {
            return PEAR::raiseError(_("This file is no MP3 audio file."));
        }

        /* Instantiate VFS object. */
        $vfs_params = Horde::getVFSConfig('mp3_vfs');
        if (is_a($vfs_params, 'PEAR_Error')) {
            return $vfs_params;
        }
        require_once 'VFS.php';
        $vfs = &VFS::singleton($vfs_params['type'], $vfs_params['params']);

        /* Store file. */
        $result = $vfs->write(MOPPLE_VFS, $id, $file['tmp_name'], true);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        /* Read MP3 ID. */
        require_once 'MP3/Id.php';
        $mp3id = &new MP3_Id(true);
        $result = $mp3id->read($file['tmp_name']);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $tags = array();
        foreach (array('name', 'artists', 'album', 'year', 'comment', 'track',
                       'genre', 'genreno', 'mpeg_ver', 'layer', 'bitrate',
                       'frequency', 'mode', 'copyright', 'length') as $tag) {
            $tags[$tag] = $mp3id->getTag($tag, null);
        }

        return $tags;
    }

    /**
     * Returns a file's content from Mopple's VFS backend.
     *
     * @param int $id  The MP3 ID.
     *
     * @return string  The file content.
     */
    function &getFile($id)
    {
        global $conf;

        /* Instantiate VFS object. */
        $vfs_params = Horde::getVFSConfig('mp3_vfs');
        if (is_a($vfs_params, 'PEAR_Error')) {
            return $vfs_params;
        }
        require_once 'VFS.php';
        $vfs = &VFS::singleton($vfs_params['type'], $vfs_params['params']);

        /* Get file. */
        return $vfs->read(MOPPLE_VFS, $id);
    }

    /**
     * Returns whether a MP3 exists for a given ID.
     *
     * @param int $id  The MP3 ID.
     *
     * @return bool  True if the MP3 exists.
     */
    function fileExists($id)
    {
        global $conf;

        /* Instantiate VFS object. */
        $vfs_params = Horde::getVFSConfig('mp3_vfs');
        if (is_a($vfs_params, 'PEAR_Error')) {
            return $vfs_params;
        }
        require_once 'VFS.php';
        $vfs = &VFS::singleton($vfs_params['type'], $vfs_params['params']);

        /* Get file. */
        return $vfs->exists(MOPPLE_VFS, $id);
    }

    /**
     * Converts a tags hash into a HTML table.
     *
     * @param array $tags  A hash with MP3 ID information.
     *
     * @return string  HTML code with the tabular representation of the tags.
     */
    function tagsToHTML($tags)
    {
        $html = '<table>';
        foreach ($tags as $tag => $value) {
            if (!is_null($value)) {
                $html .= sprintf('<tr><td>%s</td><td>%s</td></tr>',
                                 $tag,
                                 $value);
            }
        }
        $html .= '</table>';

        return $html;
    }

    /**
     * Returns the HTML code that displays an object to play a certain MP3.
     *
     * @param int $id  The MP3 ID.
     *
     * @return string  The player HTML code.
     */
    function playerHTML($id)
    {
        if (!Mopple::fileExists($id)) {
            return '';
        }
        $url = Horde::applicationUrl('download.php');
        $url = Horde::downloadUrl($id . '.mp3', array('id' => $id), $url);
        return '<object data="' . $url . '" type="audio/mpeg" width="300" height="70"><embed src="' . $url . '" /></object>';
    }

    /**
     * Build Mopple's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $conf, $registry, $browser, $print_link;

        require_once 'Horde/Menu.php';

        $menu = &new Menu(HORDE_MENU_MASK_ALL);
        $menu->add(Horde::applicationUrl('list.php'), _("List"), 'mopple.png');
        $menu->add(Horde::applicationUrl('edit.php'), _("Add"), 'mopple.png');

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
